@@ -9,49 +9,61 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 #-sb-assembling ; avoid redefinition warning
 (progn
-(defun !both-fixnum-p (temp x y)
+(defun both-fixnum-p (temp x y)
   (inst mov :dword temp x)
   (inst or :dword temp y)
   (inst test :byte temp fixnum-tag-mask))
 
-(defun !some-fixnum-p (temp x y)
+(defun some-fixnum-p (temp x y)
   (inst mov :dword temp x)
   (inst and :dword temp y)
   (inst test :byte temp fixnum-tag-mask))
 
-(defun !static-fun-addr (name)
-  #!+immobile-code (make-fixup name :static-call)
-  #!-immobile-code (ea (+ nil-value (static-fun-offset name))))
+(defun static-fun-addr (name)
+  #+immobile-code (make-fixup name :static-call)
+  #-immobile-code (ea (+ nil-value (static-fun-offset name))))
 
-(defun !call-static-fun (fun arg-count)
+(defun call-static-fun (fun arg-count)
   (inst push rbp-tn)
   (inst mov rbp-tn rsp-tn)
   (inst sub rsp-tn (* n-word-bytes 2))
   (inst mov (ea rsp-tn) rbp-tn)
   (inst mov rbp-tn rsp-tn)
   (inst mov rcx-tn (fixnumize arg-count))
-  (inst call (!static-fun-addr fun))
+  (inst call (static-fun-addr fun))
   (inst pop rbp-tn))
 
-(defun !tail-call-static-fun (fun arg-count)
+(defun tail-call-static-fun (fun arg-count)
   (inst push rbp-tn)
   (inst mov rbp-tn rsp-tn)
   (inst sub rsp-tn n-word-bytes)
   (inst push (ea (frame-byte-offset return-pc-save-offset) rbp-tn))
   (inst mov rcx-tn (fixnumize arg-count))
-  (inst jmp (!static-fun-addr fun))))
+  (inst jmp (static-fun-addr fun))))
 
 
 ;;;; addition, subtraction, and multiplication
 
+#+sb-assembling
+(defun return-single-word-bignum (dest alloc-tn source)
+  (instrument-alloc 16 nil)
+  (let ((header (logior (ash 1 n-widetag-bits) bignum-widetag)))
+    (pseudo-atomic ()
+      (allocation alloc-tn 16 nil nil 0)
+      (storew* header alloc-tn 0 0 t)
+      (storew source alloc-tn bignum-digits-offset 0)
+      (if (eq dest alloc-tn)
+          (inst or :byte dest other-pointer-lowtag)
+          (inst lea dest (ea other-pointer-lowtag alloc-tn))))))
+
 (macrolet ((define-generic-arith-routine ((fun cost) &body body)
              `(define-assembly-routine (,(symbolicate "GENERIC-" fun)
                                         (:cost ,cost)
-                                        (:return-style :full-call)
+                                        (:return-style :full-call-no-return)
                                         (:translate ,fun)
                                         (:policy :safe)
                                         (:save-p t))
@@ -62,7 +74,7 @@
 
                  (:temp rax unsigned-reg rax-offset)
                  (:temp rcx unsigned-reg rcx-offset))
-                (!both-fixnum-p rax x y)
+                (both-fixnum-p rax x y)
                 (inst jmp :nz DO-STATIC-FUN)    ; no - do generic
 
                 ,@body
@@ -70,50 +82,44 @@
                 (inst ret)
 
                 DO-STATIC-FUN
-                (!tail-call-static-fun ',(symbolicate "TWO-ARG-" fun) 2))))
+                (tail-call-static-fun ',(symbolicate "TWO-ARG-" fun) 2))))
 
-  #.`
   (define-generic-arith-routine (+ 10)
     (move res x)
     (inst add res y)
-    (inst jmp :no OKAY)
+    (inst jmp :o BIGNUM)
+    (inst clc) (inst ret)
+    BIGNUM
     ;; Unbox the overflowed result, recovering the correct sign from
     ;; the carry flag, then re-box as a bignum.
     (inst rcr res 1)
-    ,@(when (> n-fixnum-tag-bits 1)   ; don't shift by 0
-            '((inst sar res (1- n-fixnum-tag-bits))))
+    (when (> n-fixnum-tag-bits 1)   ; don't shift by 0
+      (inst sar res (1- n-fixnum-tag-bits)))
+    (return-single-word-bignum res rcx res))
 
-    (move rcx res)
-
-    (fixed-alloc res bignum-widetag (1+ bignum-digits-offset) nil)
-    (storew rcx res bignum-digits-offset other-pointer-lowtag)
-
-    OKAY)
-
-  #.`
   (define-generic-arith-routine (- 10)
     (move res x)
     (inst sub res y)
-    (inst jmp :no OKAY)
+    (inst jmp :o BIGNUM)
+    (inst clc) (inst ret)
+    BIGNUM
     ;; Unbox the overflowed result, recovering the correct sign from
     ;; the carry flag, then re-box as a bignum.
     (inst cmc)                        ; carry has correct sign now
     (inst rcr res 1)
-    ,@(when (> n-fixnum-tag-bits 1)   ; don't shift by 0
-            '((inst sar res (1- n-fixnum-tag-bits))))
-
-    (move rcx res)
-
-    (fixed-alloc res bignum-widetag (1+ bignum-digits-offset) nil)
-    (storew rcx res bignum-digits-offset other-pointer-lowtag)
-    OKAY)
+    (when (> n-fixnum-tag-bits 1)   ; don't shift by 0
+      (inst sar res (1- n-fixnum-tag-bits)))
+    (return-single-word-bignum res rcx res))
 
   (define-generic-arith-routine (* 30)
     (move rax x)                     ; must use eax for 64-bit result
     (inst sar rax n-fixnum-tag-bits) ; remove *8 fixnum bias
     (inst imul y)                    ; result in edx:eax
-    (inst jmp :no OKAY)              ; still fixnum
+    (inst jmp :o BIGNUM)
+    (move res rax)
+    (inst clc) (inst ret)
 
+    BIGNUM
     (inst shrd rax x n-fixnum-tag-bits) ; high bits from edx
     (inst sar x n-fixnum-tag-bits)      ; now shift edx too
 
@@ -122,26 +128,19 @@
     (inst cmp x rcx)
     (inst jmp :e SINGLE-WORD-BIGNUM)
 
-    (fixed-alloc res bignum-widetag (+ bignum-digits-offset 2) nil)
+    (alloc-other res bignum-widetag (+ bignum-digits-offset 2) nil)
     (storew rax res bignum-digits-offset other-pointer-lowtag)
     (storew rcx res (1+ bignum-digits-offset) other-pointer-lowtag)
-    (inst jmp DONE)
+    (inst clc) (inst ret)
 
     SINGLE-WORD-BIGNUM
-
-    (fixed-alloc res bignum-widetag (1+ bignum-digits-offset) nil)
-    (storew rax res bignum-digits-offset other-pointer-lowtag)
-    (inst jmp DONE)
-
-    OKAY
-    (move res rax)
-    DONE))
+    (return-single-word-bignum res res rax)))
 
 ;;;; negation
 
 (define-assembly-routine (generic-negate
                           (:cost 10)
-                          (:return-style :full-call)
+                          (:return-style :full-call-no-return)
                           (:policy :safe)
                           (:translate %negate)
                           (:save-p t))
@@ -149,21 +148,17 @@
                           (:res res (descriptor-reg any-reg) rdx-offset)
                           (:temp rcx unsigned-reg rcx-offset))
   (inst test :byte x fixnum-tag-mask)
-  (inst jmp :z FIXNUM)
-
-  (!tail-call-static-fun '%negate 1)
-
-  FIXNUM
+  (inst jmp :nz GENERIC)
   (move res x)
   (inst neg res)                        ; (- most-negative-fixnum) is BIGNUM
-  (inst jmp :no OKAY)
+  (inst jmp :o BIGNUM)
+  (inst clc) (inst ret)
+  BIGNUM
   (inst shr res n-fixnum-tag-bits)      ; sign bit is data - remove type bits
-  (move rcx res)
-
-  (fixed-alloc res bignum-widetag (1+ bignum-digits-offset) nil)
-  (storew rcx res bignum-digits-offset other-pointer-lowtag)
-
-  OKAY)
+  (return-single-word-bignum res rcx res)
+  (inst clc) (inst ret)
+  GENERIC
+  (tail-call-static-fun '%negate 1))
 
 ;;;; comparison
 
@@ -179,14 +174,14 @@
 
                    (:temp rcx unsigned-reg rcx-offset))
 
-                (!both-fixnum-p rcx x y)
+                (both-fixnum-p rcx x y)
                 (inst jmp :nz DO-STATIC-FUN)
 
                 (inst cmp x y)
                 (inst ret)
 
                 DO-STATIC-FUN
-                (!call-static-fun ',static-fn 2)
+                (call-static-fun ',static-fn 2)
                 ;; HACK: We depend on NIL having the lowest address of all
                 ;; static symbols (including T)
                 ,@(ecase test
@@ -196,28 +191,6 @@
   (define-cond-assem-rtn generic-< < two-arg-< :l)
   (define-cond-assem-rtn generic-> > two-arg-> :g))
 
-(define-assembly-routine (generic-eql
-                          (:translate eql)
-                          (:policy :safe)
-                          (:save-p t)
-                          (:conditional :e)
-                          (:cost 10))
-                         ((:arg x (descriptor-reg any-reg) rdx-offset)
-                          (:arg y (descriptor-reg any-reg) rdi-offset)
-
-                          (:temp rcx unsigned-reg rcx-offset))
-
-  (!some-fixnum-p rcx x y)
-  (inst jmp :nz DO-STATIC-FUN)
-
-  ;; At least one fixnum
-  (inst cmp x y)
-  (inst ret)
-
-  DO-STATIC-FUN
-  (!call-static-fun 'eql 2)
-  (inst cmp x (+ nil-value (static-symbol-offset t))))
-
 (define-assembly-routine (generic-=
                           (:translate =)
                           (:policy :safe)
@@ -228,7 +201,7 @@
                           (:arg y (descriptor-reg any-reg) rdi-offset)
 
                           (:temp rcx unsigned-reg rcx-offset))
-  (!both-fixnum-p rcx x y)
+  (both-fixnum-p rcx x y)
   (inst jmp :nz DO-STATIC-FUN)
 
   ;; Both fixnums
@@ -236,7 +209,7 @@
   (inst ret)
 
   DO-STATIC-FUN
-  (!call-static-fun 'two-arg-= 2)
+  (call-static-fun 'two-arg-= 2)
   (inst cmp x (+ nil-value (static-symbol-offset t))))
 
 #+sb-assembling
@@ -400,7 +373,7 @@
   (inst cmp rcx (ea (- other-pointer-lowtag) y))
   (inst jmp :ne done)
   (inst shr rcx n-widetag-bits)
-  ;; can you have 0 payload words? probably not, but let's be safe here.
+  ;; can you have 0 payload words? Probably not, but let's be safe here.
   (inst jrcxz done)
   loop
   (inst mov rax (ea (- other-pointer-lowtag) x rcx 8))
