@@ -213,8 +213,8 @@ an error in that case."
   (thread-%ephemeral-p thread))
 
 ;; A thread is eligible for gc iff it has finished and there are no
-;; more references to it. This list is supposed to keep a reference to
-;; all running threads.
+;; more references to it. This structure keeps a reference to
+;; all running threads ordered by stack base address.
 (sb-ext:define-load-time-global *all-threads* ())
 (sb-ext:define-load-time-global *all-threads-lock* (make-mutex :name "all threads lock"))
 
@@ -228,12 +228,15 @@ an error in that case."
   "Return a list of the live threads. Note that the return value is
 potentially stale even before the function returns, as new threads may be
 created and old ones may exit at any time."
-  (with-all-threads-lock
-    (copy-list *all-threads*)))
+  ;; No lock needed, just an atomic read, since tree mutations can't happen.
+  ;; Of course by the time we're done collecting nodes, the tree can have
+  ;; been replaced by a different tree.
+  (barrier (:read))
+  (avltree-list *all-threads*))
 
 ;;; used by debug-int.lisp to access interrupt contexts
 
-#!-sb-thread
+#-sb-thread
 (progn
   (declaim (inline sb-vm::current-thread-offset-sap))
   (defun sb-vm::current-thread-offset-sap (n)
@@ -242,48 +245,40 @@ created and old ones may exit at any time."
 
 (declaim (inline current-thread-sap))
 (defun current-thread-sap ()
-  #!+sb-thread
+  #+sb-thread
   (sb-vm::current-thread-offset-sap sb-vm::thread-this-slot)
-  #!-sb-thread
+  #-sb-thread
   (int-sap 0))
 
 (declaim (inline current-thread-os-thread))
 (defun current-thread-os-thread ()
-  #!+sb-thread
+  #+sb-thread
   (sap-int (sb-vm::current-thread-offset-sap sb-vm::thread-os-thread-slot))
-  #!-sb-thread
+  #-sb-thread
   0)
 
-(sb-ext:define-load-time-global *stack-addr-table* nil)
 (sb-ext:define-load-time-global *initial-thread* nil)
 (sb-ext:define-load-time-global *make-thread-lock* nil)
 
-(defun note-stack-range (thread table)
-  (let ((start (get-lisp-obj-address sb-vm:*control-stack-start*))
-        (end (get-lisp-obj-address sb-vm:*control-stack-end*)))
-    (multiple-value-bind (new-tree insertedp)
-        ;; We're within the scope of all-threads-lock here.
-        (treap-insert table start (cons end thread))
-      (aver insertedp)
-      (setq *stack-addr-table* new-tree))))
-
 (defun init-initial-thread ()
   (/show0 "Entering INIT-INITIAL-THREAD")
+  ;;; FIXME: is it purposeful or accidental that we recreate some of
+  ;;; the global mutexes but not *ALL-THREADS-LOCKS* ?
   (setf sb-impl::*exit-lock* (make-mutex :name "Exit Lock")
         *make-thread-lock* (make-mutex :name "Make-Thread Lock"))
   (let ((thread (%make-thread :name "main thread"
                               :%alive-p t)))
     (setf (thread-os-thread thread) (current-thread-os-thread)
+          (thread-stack-end thread) (get-lisp-obj-address sb-vm:*control-stack-end*)
           (thread-primitive-thread thread) (sap-int (current-thread-sap))
           *initial-thread* thread
           *current-thread* thread)
-    (setq sb-impl::*treap-random-state* (make-random-state *random-state*))
     (grab-mutex (thread-result-lock *initial-thread*))
-    (note-stack-range thread nil)
     ;; Either *all-threads* is empty or it contains exactly one thread
     ;; in case we are in reinit since saving core with multiple
     ;; threads doesn't work.
-    (setq *all-threads* (list thread))))
+    (setq *all-threads*
+          (avl-insert nil (get-lisp-obj-address sb-vm:*control-stack-start*) thread))))
 
 (defun main-thread ()
   "Returns the main thread of the process."
@@ -355,14 +350,14 @@ See also: RETURN-FROM-THREAD and SB-EXT:EXIT."
 
 (define-alien-routine "kill_safely"
     int
-  (os-thread #!-alpha unsigned #!+alpha unsigned-int)
+  (os-thread #-alpha unsigned #+alpha unsigned-int)
   (signal int))
 
 (define-alien-routine "wake_thread"
     int
   (os-thread unsigned))
 
-#!+sb-thread
+#+sb-thread
 (progn
   ;; FIXME it would be good to define what a thread id is or isn't
   ;; (our current assumption is that it's a fixnum).  It so happens
@@ -380,7 +375,7 @@ See also: RETURN-FROM-THREAD and SB-EXT:EXIT."
   (defun block-deferrable-signals ()
     (%block-deferrable-signals 0 0))
 
-  #!+sb-futex
+  #+sb-futex
   (progn
     (declaim (inline futex-wait %futex-wait futex-wake))
 
@@ -424,7 +419,7 @@ See also: RETURN-FROM-THREAD and SB-EXT:EXIT."
 (setf (documentation 'make-mutex 'function) "Create a mutex."
       (documentation 'mutex-name 'function) "The name of the mutex. Setfable.")
 
-#!+(and sb-thread sb-futex)
+#+(and sb-thread sb-futex)
 (progn
   (locally (declare (sb-ext:muffle-conditions sb-ext:compiler-note))
     (define-structure-slot-addressor mutex-state-address
@@ -447,14 +442,14 @@ HOLDING-MUTEX-P."
 
 (sb-ext:define-load-time-global **deadlock-lock** nil)
 
-#!+(or (not sb-thread) sb-futex)
+#+(or (not sb-thread) sb-futex)
 (defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null string))
-  #!+(and sb-thread sb-futex)
+  #+(and sb-thread sb-futex)
   (token nil))
 
-#!+(and sb-thread (not sb-futex))
+#+(and sb-thread (not sb-futex))
 (defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null string))
@@ -568,9 +563,9 @@ HOLDING-MUTEX-P."
       (loop with max-ticks = (max 100000 (min (* 2 try-ticks)
                                               (expt 10 7)))
             for scale of-type fixnum = 1
-            then (let ((x (logand most-positive-fixnum (* 2 scale))))
+            then (let ((x (logand sb-xc:most-positive-fixnum (* 2 scale))))
                    (if (> scale x)
-                       most-positive-fixnum
+                       sb-xc:most-positive-fixnum
                        x))
             do (try)
                (let* ((now (get-tick))
@@ -632,13 +627,13 @@ returns NIL each time."
             (,deadline
               (when ,sec
                 (+ (get-internal-real-time)
-                   (round (* ,seconds internal-time-units-per-second))))))
+                   (round (* ,seconds sb-xc:internal-time-units-per-second))))))
        (flet ((,name ()
                 (when ,deadline
                   (let ((,time-left (- ,deadline (get-internal-real-time))))
                     (if (plusp ,time-left)
                         (* (coerce ,time-left 'single-float)
-                           (load-time-value (/ 1.0f0 internal-time-units-per-second) t))
+                           (sb-xc:/ $1.0f0 sb-xc:internal-time-units-per-second))
                         0)))))
          ,@body))))
 
@@ -648,14 +643,14 @@ returns NIL each time."
   (let ((old (mutex-%owner mutex)))
     (when (eq new-owner old)
       (error "Recursive lock attempt ~S." mutex))
-    #!-sb-thread
+    #-sb-thread
     (when old
       (error "Strange deadlock on ~S in an unithreaded build?" mutex))
-    #!-(and sb-thread sb-futex)
+    #-(and sb-thread sb-futex)
     (and (not old)
          ;; Don't even bother to try to CAS if it looks bad.
          (not (sb-ext:compare-and-swap (mutex-%owner mutex) nil new-owner)))
-    #!+(and sb-thread sb-futex)
+    #+(and sb-thread sb-futex)
     ;; From the Mutex 2 algorithm from "Futexes are Tricky" by Ulrich Drepper.
     (when (eql +lock-free+ (sb-ext:compare-and-swap (mutex-state mutex)
                                                     +lock-free+
@@ -665,13 +660,13 @@ returns NIL each time."
           (bug "Old owner in free mutex: ~S" prev))
         t))))
 
-#!+sb-thread
+#+sb-thread
 (defun %%wait-for-mutex (mutex new-owner to-sec to-usec stop-sec stop-usec)
   (declare (type mutex mutex) (optimize (speed 3)))
   (declare (sb-ext:muffle-conditions sb-ext:compiler-note))
-  #!-sb-futex
+  #-sb-futex
   (declare (ignore to-sec to-usec))
-  #!-sb-futex
+  #-sb-futex
   (flet ((cas ()
            (loop repeat 100
                  when (and (progn
@@ -687,7 +682,7 @@ returns NIL each time."
            (with-interrupts nil)))
     (declare (dynamic-extent #'cas))
     (%%wait-for #'cas stop-sec stop-usec))
-  #!+sb-futex
+  #+sb-futex
   ;; This is a fairly direct translation of the Mutex 2 algorithm from
   ;; "Futexes are Tricky" by Ulrich Drepper.
   (flet ((maybe (old)
@@ -727,7 +722,7 @@ returns NIL each time."
        ;; Spin.
        (go :retry))))
 
-#!+sb-thread
+#+sb-thread
 (defun %wait-for-mutex (mutex self timeout to-sec to-usec stop-sec stop-usec deadlinep)
   (declare (sb-ext:muffle-conditions sb-ext:compiler-note))
   (with-deadlocks (self mutex timeout)
@@ -748,7 +743,7 @@ returns NIL each time."
   (declare (ignorable waitp timeout))
   (let ((new-owner (or new-owner *current-thread*)))
     (or (%try-mutex mutex new-owner)
-        #!+sb-thread
+        #+sb-thread
         (when waitp
           (multiple-value-call #'%wait-for-mutex
             mutex new-owner timeout (decode-timeout timeout))))))
@@ -791,7 +786,7 @@ Notes:
   (declare (ignorable waitp timeout))
   (let ((self *current-thread*))
     (or (%try-mutex mutex self)
-        #!+sb-thread
+        #+sb-thread
         (when waitp
           (multiple-value-call #'%wait-for-mutex
             mutex self timeout (decode-timeout timeout))))))
@@ -821,7 +816,7 @@ IF-NOT-OWNER is :FORCE)."
       ;; FIXME: Is a :memory barrier too strong here?  Can we use a :write
       ;; barrier instead?
       (barrier (:memory)))
-    #!+(and sb-thread sb-futex)
+    #+(and sb-thread sb-futex)
     (when old-owner
       ;; FIXME: once ATOMIC-INCF supports struct slots with word sized
       ;; unsigned-byte type this can be used:
@@ -843,7 +838,7 @@ IF-NOT-OWNER is :FORCE)."
 
 ;;;; Waitqueues/condition variables
 
-#!+(and sb-thread (not sb-futex))
+#+(and sb-thread (not sb-futex))
 (progn
   (defun %waitqueue-enqueue (thread queue)
     (setf (thread-waiting-for thread) queue)
@@ -897,7 +892,7 @@ IF-NOT-OWNER is :FORCE)."
 (setf (documentation 'waitqueue-name 'function) "The name of the waitqueue. Setfable."
       (documentation 'make-waitqueue 'function) "Create a waitqueue.")
 
-#!+(and sb-thread sb-futex)
+#+(and sb-thread sb-futex)
 (locally (declare (sb-ext:muffle-conditions sb-ext:compiler-note))
   (define-structure-slot-addressor waitqueue-token-address
       :structure waitqueue
@@ -906,11 +901,11 @@ IF-NOT-OWNER is :FORCE)."
 (declaim (inline %condition-wait))
 (defun %condition-wait (queue mutex
                         timeout to-sec to-usec stop-sec stop-usec deadlinep)
-  #!-sb-thread
+  #-sb-thread
   (declare (ignore queue mutex to-sec to-usec stop-sec stop-usec deadlinep))
-  #!-sb-thread
+  #-sb-thread
   (sb-ext:wait-for nil :timeout timeout) ; Yeah...
-  #!+sb-thread
+  #+sb-thread
   (let ((me *current-thread*))
     (barrier (:read))
     (unless (eq me (mutex-%owner mutex))
@@ -921,7 +916,7 @@ IF-NOT-OWNER is :FORCE)."
       (without-interrupts
         (unwind-protect
              (progn
-               #!-sb-futex
+               #-sb-futex
                (progn
                  (%with-cas-lock ((waitqueue-%owner queue))
                    (%waitqueue-enqueue me queue))
@@ -935,7 +930,7 @@ IF-NOT-OWNER is :FORCE)."
                              (allow-with-interrupts
                                (%%wait-for #'wakeup stop-sec stop-usec)))
                            :timeout)))
-               #!+sb-futex
+               #+sb-futex
                (with-pinned-objects (queue me)
                  (setf (waitqueue-token queue) me)
                  (release-mutex mutex)
@@ -960,7 +955,7 @@ IF-NOT-OWNER is :FORCE)."
                           ;;  0 = normal wakeup
                           ;;  2 = EINTR, a spurious wakeup
                           :ok)))))
-          #!-sb-futex
+          #-sb-futex
           (%with-cas-lock ((waitqueue-%owner queue))
             (if (eq queue (thread-waiting-for me))
                 (%waitqueue-drop me queue)
@@ -1066,18 +1061,18 @@ associated data:
 
 IMPORTANT: The same mutex that is used in the corresponding CONDITION-WAIT
 must be held by this thread during this call."
-  #!-sb-thread
+  #-sb-thread
   (declare (ignore queue n))
-  #!-sb-thread
+  #-sb-thread
   (error "Not supported in unithread builds.")
-  #!+sb-thread
+  #+sb-thread
   (declare (type (and fixnum (integer 1)) n))
-  #!+sb-thread
+  #+sb-thread
   (progn
-    #!-sb-futex
+    #-sb-futex
     (with-cas-lock ((waitqueue-%owner queue))
       (%waitqueue-wakeup queue n))
-    #!+sb-futex
+    #+sb-futex
     (progn
     ;; No problem if >1 thread notifies during the comment in condition-wait:
     ;; as long as the value in queue-data isn't the waiting thread's id, it
@@ -1100,7 +1095,7 @@ must be held by this thread during this call."
                     ;; On a 64-bit platform truncating M-P-F to an int
                     ;; results in -1, which wakes up only one thread.
                     (ldb (byte 29 0)
-                         most-positive-fixnum)))
+                         sb-xc:most-positive-fixnum)))
 
 
 ;;;; Semaphores
@@ -1308,6 +1303,7 @@ on this semaphore, then N of them is woken up."
   (with-session-lock (session)
     (let ((was-foreground (eq thread (foreground-thread session))))
       (setf (session-threads session)
+            ;; FIXME: I assume these could use DELQ1.
             ;; DELQ never conses, but DELETE does. (FIXME)
             (delq thread (session-threads session))
             (session-interactive-threads session)
@@ -1327,18 +1323,17 @@ on this semaphore, then N of them is woken up."
       (call-with-new-session (function ,fb-name)))))
 
 ;;; Remove thread from its session, if it has one.
-#!+sb-thread
+#+sb-thread
 (defun handle-thread-exit (thread control-stack-start)
   (/show0 "HANDLING THREAD EXIT")
   (when *exit-in-process*
     (%exit))
   ;; Lisp-side cleanup
   (with-all-threads-lock
-    (setq *stack-addr-table* (treap-delete control-stack-start *stack-addr-table*))
     (setf (thread-%alive-p thread) nil)
     (setf (thread-os-thread thread)
           (ldb (byte sb-vm:n-word-bits 0) -1))
-    (setq *all-threads* (delq1 thread *all-threads*))
+    (setq *all-threads* (avl-delete control-stack-start *all-threads*))
     (when *session*
       (%delete-thread-from-session thread *session*))))
 
@@ -1422,17 +1417,23 @@ thread is not the foreground thread."
   "Returns T if thread had been running in background, NIL if it was
 interactive."
   (declare (ignore stream))
-  #!-sb-thread nil
-  #!+sb-thread
+  #-sb-thread nil
+  #+sb-thread
   (prog1
       (with-session-lock (*session*)
-        (not (member *current-thread*
-                     (session-interactive-threads *session*))))
+        (let ((foreground (foreground-thread)))
+          (unless (or (null foreground)
+                      (eq foreground *current-thread*))
+            (format *error-output* "~%The current thread is not at the foreground,~@
+SB-THREAD:RELEASE-FOREGROUND has to be called in ~s~%for this thread to enter the debugger.~%"
+                    foreground))
+          (not (member *current-thread*
+                       (session-interactive-threads *session*)))))
     (get-foreground)))
 
 (defun get-foreground ()
-  #!-sb-thread t
-  #!+sb-thread
+  #-sb-thread t
+  #+sb-thread
   (let ((session *session*)
         (was-foreground t))
     (loop
@@ -1460,9 +1461,9 @@ interactive."
 (defun release-foreground (&optional next)
   "Background this thread.  If NEXT is supplied, arrange for it to
 have the foreground next."
-  #!-sb-thread (declare (ignore next))
-  #!-sb-thread nil
-  #!+sb-thread
+  #-sb-thread (declare (ignore next))
+  #-sb-thread nil
+  #+sb-thread
   (let ((session *session*))
     (with-session-lock (session)
       (symbol-macrolet
@@ -1484,6 +1485,7 @@ session."
 session."
   (first (interactive-threads session)))
 
+#-win32
 (defun make-listener-thread (tty-name)
   (aver (probe-file tty-name))
   (let* ((in (sb-unix:unix-open (namestring tty-name) sb-unix:o_rdwr #o666))
@@ -1492,18 +1494,18 @@ session."
     (labels ((thread-repl ()
                (sb-unix::unix-setsid)
                (let* ((sb-impl::*stdin*
-                       (make-fd-stream in :input t :buffering :line
-                                       :dual-channel-p t))
+                        (make-fd-stream in :input t :buffering :line
+                                           :dual-channel-p t))
                       (sb-impl::*stdout*
-                       (make-fd-stream out :output t :buffering :line
-                                              :dual-channel-p t))
+                        (make-fd-stream out :output t :buffering :line
+                                            :dual-channel-p t))
                       (sb-impl::*stderr*
-                       (make-fd-stream err :output t :buffering :line
-                                              :dual-channel-p t))
+                        (make-fd-stream err :output t :buffering :line
+                                            :dual-channel-p t))
                       (sb-impl::*tty*
-                       (make-fd-stream err :input t :output t
-                                              :buffering :line
-                                              :dual-channel-p t))
+                        (make-fd-stream err :input t :output t
+                                            :buffering :line
+                                            :dual-channel-p t))
                       (sb-impl::*descriptor-handlers* nil))
                  (with-new-session ()
                    (unwind-protect
@@ -1514,34 +1516,32 @@ session."
 
 ;;;; The beef
 
-;;; One must parse this name carefully: it is the initial "thread function trampoline",
-;;; and not the "initial thread" "function trampoline".
-;;; i.e. there is an initial thread, which DOES NOT start via this function.
-;;; All threads other than the initial thread DO start via this function.
-;;; The initial thread has its own way of doing things, which ends up calling
-;;; INIT-INITIAL-THREAD.  It might be nice to come up with some better naming.
-#!+sb-thread
-(defun initial-thread-function-trampoline (thread setup-sem real-function arguments)
+;;; All threads other than the initial thread start via this function.
+#+sb-thread
+(defun new-lisp-thread-trampoline (thread setup-sem real-function arguments)
   ;; Can't initiate GC before *current-thread* is set, otherwise the
   ;; locks grabbed by SUB-GC wouldn't function.
   ;; Other threads can GC with impunity.
   (setf *current-thread* thread ; is thread-local already
         (thread-os-thread thread) (current-thread-os-thread)
+        (thread-stack-end thread) (get-lisp-obj-address sb-vm:*control-stack-end*)
         (thread-primitive-thread thread) (sap-int (current-thread-sap)))
   ;; *ALLOC-SIGNAL* is made thread-local by create_thread_struct()
   ;; so this assigns into TLS, not the global value.
   (setf sb-vm:*alloc-signal* *default-alloc-signal*)
   (with-mutex ((thread-result-lock thread))
-    (let* ((thread-list (list thread thread))
-           (session-cons (cdr thread-list)))
-      (with-all-threads-lock
-        (note-stack-range thread *stack-addr-table*)
-        (setf (cdr thread-list) *all-threads*
-              *all-threads* thread-list))
-      (let ((session *session*))
+    (with-all-threads-lock
+        (let ((addr (get-lisp-obj-address sb-vm:*control-stack-start*)))
+          ;; If ADDR exists, then we have a bug in the thread exit handler.
+          ;; The workaround here would be to delete the old thread first,
+          ;; but I'd rather find out about the bug than bury it.
+          (aver (not (avl-find addr *all-threads*)))
+          (setq *all-threads* (avl-insert *all-threads* addr thread))))
+    (let ((session *session*)
+          (session-cons (list thread)))
         (with-session-lock (session)
           (setf (cdr session-cons) (session-threads session)
-                (session-threads session) session-cons))))
+                (session-threads session) session-cons)))
     (setf (thread-%alive-p thread) t)
 
     (when setup-sem
@@ -1577,7 +1577,7 @@ session."
                                         #'sb-di::catch-runaway-unwind))
                                   (when *exit-in-process*
                                     (sb-impl::call-exit-hooks))))
-                             #!+sb-safepoint
+                             #+sb-safepoint
                              (sb-kernel::gc-safepoint))))
                 ;; we're going down, can't handle interrupts
                 ;; sanely anymore. gc remains enabled.
@@ -1588,7 +1588,7 @@ session."
                 ;; interupts to be lost: sigint comes to
                 ;; mind.
                 (setq *interrupt-pending* nil)
-                #!+sb-thruption
+                #+sb-thruption
                 (setq *thruption-pending* nil)
                 (handle-thread-exit thread
                                     (get-lisp-obj-address
@@ -1605,33 +1605,58 @@ Invoking the initial ABORT restart established by MAKE-THREAD
 terminates the thread.
 
 See also: RETURN-FROM-THREAD, ABORT-THREAD."
-  #!-sb-thread (declare (ignore function name arguments ephemeral))
-  #!-sb-thread (error "Not supported in unithread builds.")
-  #!+sb-thread (assert (or (atom arguments)
-                           (null (cdr (last arguments))))
-                       (arguments)
-                       "Argument passed to ~S, ~S, is an improper list."
-                       'make-thread arguments)
-  #!+sb-thread
-  (let ((thread (%make-thread :name name :%ephemeral-p ephemeral)))
+  #-sb-thread (declare (ignore function name arguments ephemeral))
+  #-sb-thread (error "Not supported in unithread builds.")
+  #+sb-thread
+  (progn (assert (or (atom arguments)
+                     (null (cdr (last arguments))))
+                 (arguments)
+                 "Argument passed to ~S, ~S, is an improper list."
+                 'make-thread arguments)
+         (run-thread (%make-thread :name name :%ephemeral-p ephemeral)
+                     function arguments)))
+
+;;; The purpose of splitting out RUN-THREAD from MAKE-THREAD is that when
+;;; starting the finalizer thread, we might be able to do:
+;;; (let ((thread (%make-thread :name "finalizer" :%ephemeral-p t)))
+;;;   (when (cas *finalizer-thread* nil thread)
+;;;     (run-thread thread ...)
+;;; which is possibly an improvement in two ways:
+
+;;; (1) it ensures that there is no hidden state in the transition diagram
+;;;     when we are invisibly starting the finalizer thread but have not made it
+;;;     known to FINALIZER-THREAD-STOP that we are doing so. There would be a
+;;;     thread object published or not - and no "maybe starting" state.
+;;; (2) imagine two threads, each of which actually GC'd - so the 'gc_happened'
+;;;     flag in gc-common.c is T for both - and each wants to start the finalizer.
+;;;     They both get all the way into NEW-LISP-THREAD-TRAMPOLINE, only for one
+;;;     to lose the CAS on *FINALIZER-THREAD*. It's a lot of overhead to start
+;;;     a thread that does nothing and then exits.
+;;;
+;;; But it's not all fun and games, because we'd have to figure out how to
+;;; get FINALIZER-THREAD-STOP _not_ to attempt to join a thread that has not
+;;; yet sprung into being as an OS-level thread.
+
+#+sb-thread
+(defun run-thread (thread function arguments)
     (declare (inline make-semaphore
                      make-waitqueue
                      make-mutex))
     (let* ((setup-sem (make-semaphore :name "Thread setup semaphore"))
            (real-function (coerce function 'function))
            (arguments     (ensure-list arguments))
-           #!+(or win32 darwin)
+           #+(or win32 darwin)
            (fp-modes (dpb 0 sb-vm:float-sticky-bits ;; clear accrued bits
                           (sb-vm:floating-point-modes))))
       (declare (dynamic-extent setup-sem))
-      (dx-flet ((initial-thread-function ()
+      (dx-flet ((start-routine ()
                   ;; Inherit parent thread's FP modes
-                  #!+(or win32 darwin)
+                  #+(or win32 darwin)
                   (setf (sb-vm:floating-point-modes) fp-modes)
                   ;; As it is, this lambda must not cons until we are
                   ;; ready to run GC. Be careful.
-                  (initial-thread-function-trampoline thread setup-sem
-                                                      real-function arguments)))
+                  (new-lisp-thread-trampoline thread setup-sem
+                                              real-function arguments)))
         ;; Holding mutexes or waiting on sempahores inside WITHOUT-GCING will lock up
         (aver (not *gc-inhibit*))
         ;; Keep INITIAL-FUNCTION in the dynamic extent until the child
@@ -1640,11 +1665,10 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
         ;; INITIAL-FUNCTION to another thread.
         ;; (Does WITHOUT-INTERRUPTS really matter now that it's DXed?)
         (with-system-mutex (*make-thread-lock*)
-          (if (zerop
-               (%create-thread (get-lisp-obj-address #'initial-thread-function)))
+          (if (zerop (%create-thread (get-lisp-obj-address #'start-routine)))
               (setf thread nil)
               (wait-on-semaphore setup-sem)))))
-    (or thread (error "Could not create a new thread."))))
+    (or thread (error "Could not create a new thread.")))
 
 (defun join-thread (thread &key (default nil defaultp) timeout)
   "Suspend current thread until THREAD exits. Return the result values
@@ -1708,7 +1732,7 @@ subject to change."
      ,@body))
 
 ;;; Called from the signal handler.
-#!-(or sb-thruption win32)
+#-(or sb-thruption win32)
 (defun run-interruption ()
   (let ((interruption (with-interruptions-lock (*current-thread*)
                         (pop (thread-interruptions *current-thread*)))))
@@ -1721,7 +1745,7 @@ subject to change."
     (when interruption
       (funcall interruption))))
 
-#!+sb-thruption
+#+sb-thruption
 (defun run-interruption (*current-internal-error-context*)
   (in-interruption () ;the non-thruption code does this in the signal handler
     (let ((interruption (with-interruptions-lock (*current-thread*)
@@ -1800,12 +1824,12 @@ the state of a thread:
   (interrupt-thread thread #'break)
 
 Short version: be careful out there."
-  #!+(and (not sb-thread) win32)
-  #!+(and (not sb-thread) win32)
+  #+(and (not sb-thread) win32)
+  #+(and (not sb-thread) win32)
   (declare (ignore thread))
   (with-interrupt-bindings
     (with-interrupts (funcall function)))
-  #!-(and (not sb-thread) win32)
+  #-(and (not sb-thread) win32)
   (let ((os-thread (thread-os-thread thread)))
     (cond ((= os-thread (ldb (byte sb-vm:n-word-bits 0) -1))
            (error 'interrupt-thread-error :thread thread))
@@ -1879,12 +1903,12 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
 ;;; are an SBCL developer, are doing something that you should discuss
 ;;; with an SBCL developer first, or are doing something that you
 ;;; should probably discuss with a professional psychiatrist first
-#!+sb-thread
+#+sb-thread
 (progn
 
   (sb-ext:define-load-time-global sb-vm::*free-tls-index* 0)
   ;; Keep in sync with 'compiler/generic/parms.lisp'
-  #!+ppc ; only PPC uses a separate symbol for the TLS index lock
+  #+(or ppc ppc64) ; only PPC uses a separate symbol for the TLS index lock
   (!define-load-time-global sb-vm::*tls-index-lock* 0)
 
   (defun %symbol-value-in-thread (symbol thread)
@@ -1893,7 +1917,7 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
     (with-all-threads-lock
       (if (thread-alive-p thread)
           (let ((val (sap-ref-lispobj (int-sap (thread-primitive-thread thread))
-                                      (get-lisp-obj-address (symbol-tls-index symbol)))))
+                                      (symbol-tls-index symbol))))
             (case (get-lisp-obj-address val)
               (#.sb-vm:no-tls-value-marker-widetag (values nil :no-tls-value))
               (#.sb-vm:unbound-marker-widetag (values nil :unbound-in-thread))
@@ -1905,7 +1929,7 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
     ;; area...
     (with-all-threads-lock
       (if (thread-alive-p thread)
-          (let ((offset (get-lisp-obj-address (symbol-tls-index symbol))))
+          (let ((offset (symbol-tls-index symbol)))
             (cond ((zerop offset)
                    (values nil :no-tls-value))
                   (t
@@ -1924,7 +1948,7 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
     ;; (raw value) manifesting in Lisp as a fixnum.
     ;; The sign bit of sb-vm::*free-tls-index* is a semaphore,
     ;; except on PPC where it isn't, but masking is fine in any case.
-    (do ((index (- (ash (logand sb-vm::*free-tls-index* most-positive-fixnum)
+    (do ((index (- (ash (logand sb-vm::*free-tls-index* sb-xc:most-positive-fixnum)
                         sb-vm:n-fixnum-tag-bits)
                    sb-vm:n-word-bytes)
                 (- index sb-vm:n-word-bytes))
@@ -1952,7 +1976,7 @@ Can also be used with SETF to change the thread-local value of SYMBOL.
 SYMBOL-VALUE-IN-THREAD is primarily intended as a debugging tool, and not as a
 mechanism for inter-thread communication."
   (declare (symbol symbol) (thread thread))
-  #!+sb-thread
+  #+sb-thread
   (multiple-value-bind (res status) (%symbol-value-in-thread symbol thread)
     (if (eq :ok status)
         (values res t)
@@ -1962,7 +1986,7 @@ mechanism for inter-thread communication."
                    :thread thread
                    :info (list :read status))
             (values nil nil))))
-  #!-sb-thread
+  #-sb-thread
   (if (boundp symbol)
       (values (symbol-value symbol) t)
       (if errorp
@@ -1974,7 +1998,7 @@ mechanism for inter-thread communication."
 
 (defun (setf symbol-value-in-thread) (value symbol thread &optional (errorp t))
   (declare (symbol symbol) (thread thread))
-  #!+sb-thread
+  #+sb-thread
   (multiple-value-bind (res status) (%set-symbol-value-in-thread symbol thread value)
     (if (eq :ok status)
         (values res t)
@@ -1984,7 +2008,7 @@ mechanism for inter-thread communication."
                    :thread thread
                    :info (list :write status))
             (values nil nil))))
-  #!-sb-thread
+  #-sb-thread
   (if (boundp symbol)
       (values (setf (symbol-value symbol) value) t)
       (if errorp
