@@ -43,20 +43,50 @@
         (cons (and (find-package "SB-INTERPRETER") value1)
               value2)))))
 
-(defvar *make-host-parallelism*
-  (or #+sbcl
-      (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
-        (when envvar
-          (require :sb-posix)
-          (parse-make-host-parallelism envvar)))))
-
+(defvar *make-host-parallelism* nil)
 (defun make-host-1-parallelism () (car *make-host-parallelism*))
 (defun make-host-2-parallelism () (cdr *make-host-parallelism*))
 
 #+sbcl
-(let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
-           (if (eq access :external) sym 'sb-unix:unix-exit))))
-  (defun exit-process (arg) (funcall f arg)))
+(progn
+  (setq *make-host-parallelism*
+        (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
+          (when envvar
+            (require :sb-posix)
+            (parse-make-host-parallelism envvar))))
+  (defmacro with-subprocesses (&rest body) `(progn ,@body))
+  (let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
+             (if (eq access :external) sym 'sb-unix:unix-exit))))
+    (defun exit-process (arg) (funcall f arg))
+    (defun exit-subprocess (arg) (funcall f arg)))
+  ;; Lazily reference sb-posix because it may not be loaded
+  (defun posix-fork () (funcall (intern "FORK" "HOST-SB-POSIX")))
+  (defun getpid () (funcall (intern "UNIX-GETPID" "HOST-SB-UNIX")))
+  (defun posix-wait () (funcall (intern "WAIT" "HOST-SB-POSIX"))))
+
+#+clisp
+(progn
+  (setq *make-host-parallelism*
+        (let ((envvar (ext:getenv "SBCL_MAKE_PARALLEL")))
+          (when envvar
+            (parse-make-host-parallelism envvar))))
+  ;; FFI symbols won't exist if libffcall could not be found at build time.
+  (defmacro with-subprocesses (&rest rest)
+    (cons (or (find-symbol "WITH-SUBPROCESSES" "POSIX") 'progn) rest))
+  ;; clisp doesn't expose fork() and consequently doesn't behave
+  ;; correctly when (EXT:EXIT) is called in a forked child process.
+  (defun exit-process (arg) (ext:exit arg))
+  #+#.(cl:if (cl:find-package "FFI") '(and) '(or))
+  (progn (ffi:def-call-out exit-subprocess (:name "exit") (:arguments (arg ffi:int))
+                  (:library :default) (:language :stdc))
+         (ffi:def-call-out posix-fork (:name "fork") (:return-type ffi:int)
+                  (:library :default) (:language :stdc)))
+  (defun getpid () (posix:process-id))
+  (defun posix-wait ()
+    (multiple-value-bind (pid status code) (posix:wait)
+      (if (eql status :exited)
+          (values pid code)
+          (values pid (- code))))))
 
 ;;; If TRUE, then COMPILE-FILE is being invoked only to process
 ;;; :COMPILE-TOPLEVEL forms, not to produce an output file.
@@ -148,15 +178,11 @@
 ;;; Return an expression read from the file named NAMESTRING.
 ;;; For user-supplied inputs, protect against more than one expression
 ;;; appearing in the file. With trusted inputs we needn't bother.
-(defun read-from-file (namestring)
+(defun read-from-file (namestring &optional (enforce-single-expr t))
   (with-open-file (s (prepend-genfile-path namestring))
     (let* ((result (read s))
            (eof-result (cons nil nil)))
-      (when (string= (pathname-name namestring) "build-order")
-        ;; build-order uses both host and target conditionals.
-        ;; Our sanity checks during make-host-1 would complain
-        ;; about #+feature when reading the second expression in the file
-        ;; whenever such feature is also a possible target feature.
+      (unless enforce-single-expr
         (return-from read-from-file result))
       (unless (eq (read s nil eof-result) eof-result)
         (error "more than one expression in file ~S" namestring))
@@ -186,8 +212,7 @@
 ;;; When cross-compiling, the *FEATURES* set for the target Lisp is
 ;;; not in general the same as the *FEATURES* set for the host Lisp.
 ;;; In order to refer to target features specifically, we refer to
-;;; SB-XC:*FEATURES* instead of CL:*FEATURES*, and use the #!+ and #!-
-;;; readmacros instead of the ordinary #+ and #- readmacros.
+;;; SB-XC:*FEATURES* instead of CL:*FEATURES*.
 ;;;
 ;;; To support building in a read-only filesystem, the 'local-target-features'
 ;;; file might not be directly located here, since it's a generated file.
@@ -256,13 +281,13 @@
 (let ((feature-compatibility-tests
        '(("(and sb-thread (not gencgc))"
           ":SB-THREAD requires :GENCGC")
-         ("(and sb-thread (not (or ppc x86 x86-64 arm64)))"
+         ("(and sb-thread (not (or ppc ppc64 x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
          ("(and gencgc cheneygc)"
           ":GENCGC and :CHENEYGC are incompatible")
-         ("(and cheneygc (not (or alpha arm hppa mips ppc sparc)))"
+         ("(and cheneygc (not (or alpha arm hppa mips ppc riscv sparc)))"
           ":CHENEYGC not supported on selected architecture")
-         ("(and gencgc (not (or sparc ppc ppc64 x86 x86-64 arm arm64)))"
+         ("(and gencgc (not (or sparc ppc ppc64 x86 x86-64 arm arm64 riscv)))"
           ":GENCGC not supported on selected architecture")
          ("(not (or gencgc cheneygc))"
           "One of :GENCGC or :CHENEYGC must be enabled")
@@ -296,6 +321,11 @@
           ":IMMOBILE-CODE requires :IMMOBILE-SPACE feature")
          ("(and immobile-symbols (not immobile-space))"
           ":IMMOBILE-SYMBOLS requires :IMMOBILE-SPACE feature")
+         ("(and int4-breakpoints x86)"
+          ;; 0xCE is a perfectly good 32-bit instruction,
+          ;; unlike on x86-64 where it is illegal. It's therefore
+          ;; confusing to allow this feature in a 32-bit build.
+          ":INT4-BREAKPOINTS are incompatible with x86")
          ;; There is still hope to make multithreading on DragonFly x86-64
          ("(and sb-thread x86 dragonfly)"
           ":SB-THREAD not supported on selected architecture")))
@@ -315,11 +345,11 @@
 ;;; values of special variables such as *** and +, anyway). Set up
 ;;; machinery to warn us when/if we change it.
 ;;;
-;;; All code depending on this is itself dependent on #!+SB-SHOW.
-#!+sb-show
-(progn
+;;; All code depending on this is itself dependent on #+SB-SHOW.
+(defvar *cl-snapshot*)
+(when (member :sb-show sb-xc:*features*)
   (load "src/cold/snapshot.lisp")
-  (defvar *cl-snapshot* (take-snapshot "COMMON-LISP")))
+  (setq *cl-snapshot* (take-snapshot "COMMON-LISP")))
 
 ;;;; master list of source files and their properties
 
@@ -356,13 +386,31 @@
     ;; never figured out but which were apparently acceptable in CMU
     ;; CL. Eventually, it would be great to just get rid of all
     ;; warnings and remove support for this flag. -- WHN 19990323)
-    :ignore-failure-p))
+    :ignore-failure-p
+    ;; meaning: ignore this flag.
+    ;; This works around nonstandard behavior of "#." in certain hosts.
+    ;; When the evaluated form yields 0 values, ECL and CLISP treat it
+    ;; as though if yielded NIL:
+    ;; * (read-from-string "#(#.(cl:if (cl:eql 1 2) x (values)))")
+    ;;   => #(NIL)
+    ;; The correct value for the above expression - as obtained in SBCL,
+    ;; CCL, and ABCL - is #() because _any_ reader macro is permitted
+    ;; to produce 0 values. In fact you can demonstrate this by actually
+    ;; implementing your own "#." which conditionally returns 0 values,
+    ;; and seeing that it works in any lisp including the suspect ones.
+    ;; The oft-used idiom of "#+#.(cl:if (test) '(and) '(or)) X"
+    ;; is sufficiently unclear that its worth allowing a spurious NIL
+    ;; just to avoid that ugly mess.
+    nil))
 
+;;; The specialized array registry has file-wide scope. Hacking that aspect
+;;; into the xc build scaffold seemed slightly easier than hacking the
+;;; compiler (i.e. making the registry a slot of the fasl-output struct)
 (defvar *array-to-specialization* (make-hash-table :test #'eq))
 
-(defmacro do-stems-and-flags ((stem flags) &body body)
+(defmacro do-stems-and-flags ((stem flags build-phase) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
-    `(dolist (,stem-and-flags (get-stems-and-flags))
+    `(dolist (,stem-and-flags (get-stems-and-flags ,build-phase))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
          ,@body
@@ -371,13 +419,16 @@
 ;;; Given a STEM, remap the path component "/target/" to a suitable
 ;;; target directory.
 (defun stem-remap-target (stem)
-  (let ((position (search "/target/" stem)))
-    (if position
-      (concatenate 'string
-                   (subseq stem 0 (1+ position))
-                   (string-downcase (target-platform-keyword))
-                   (subseq stem (+ position 7)))
-      stem)))
+  (flet ((try-replacing (this that)
+           (let ((position (search this stem)))
+             (when position
+               (concatenate 'string
+                            (subseq stem 0 (1+ position))
+                            (string-downcase that)
+                            (subseq stem (+ position (length this) -1)))))))
+    (or (try-replacing "/target/" (target-platform-keyword))
+        (try-replacing "/asm-target/" (backend-assembler-target-name))
+        stem)))
 (compile 'stem-remap-target)
 
 ;;; Determine the source path for a stem by remapping from the abstract name
@@ -412,13 +463,26 @@
 (compile 'stem-object-path)
 
 (defvar *stems-and-flags* nil)
-;;; Check for stupid typos in FLAGS list keywords.
-(defun get-stems-and-flags ()
- (when *stems-and-flags*
-   (return-from get-stems-and-flags *stems-and-flags*))
- (setf *stems-and-flags* (read-from-file "build-order.lisp-expr"))
- (let ((stems (make-hash-table :test 'equal)))
-  (do-stems-and-flags (stem flags)
+;;; Read the set of files to compile with respect to a build phase, 1 or 2.
+(defun get-stems-and-flags (build-phase)
+  (when (and *stems-and-flags* (eql (car *stems-and-flags*) build-phase))
+    (return-from get-stems-and-flags (cdr *stems-and-flags*)))
+  (let* ((feature (aref #(:sb-xc-host :sb-xc) (1- build-phase)))
+         (list
+          ;; The build phase feature goes into CL:*FEATURES*, not SB-XC:*FEATURES*
+          ;; because firstly we don't use feature expressions to control the set of
+          ;; files pertinent to the build phase - that is governed by :NOT-{HOST,TARGET}
+          ;; flags, and secondly we can not assume existence of the SB-XC package in
+          ;; warm build. The sole reason for this hack is to allow testing for CMU
+          ;; as the build host in make-host-1 which apparently needs to be allowed
+          ;; to produce warnings as a bug workaround.
+          (let ((cl:*features* (cons feature cl:*features*))
+                (*readtable* *xc-readtable*))
+            (read-from-file "build-order.lisp-expr" nil))))
+    (setf *stems-and-flags* (cons build-phase list)))
+  ;; Now check for duplicate stems and bogus flags.
+  (let ((stems (make-hash-table :test 'equal)))
+    (do-stems-and-flags (stem flags build-phase)
     ;; We do duplicate stem comparison based on the object path in
     ;; order to cover the case of stems with an :assem flag, which
     ;; have two entries but separate object paths for each.  KLUDGE:
@@ -426,18 +490,19 @@
     ;; set up later in the build process and we don't actually care
     ;; what it is so long as it doesn't change while we're checking
     ;; for duplicate stems.
-    (let* ((*target-obj-prefix* "")
-           (object-path (stem-object-path stem flags :target-compile)))
-      (if (gethash object-path stems)
-          (error "duplicate stem ~S in *STEMS-AND-FLAGS*" stem)
-          (setf (gethash object-path stems) t)))
+      (let* ((*target-obj-prefix* "")
+             (object-path (stem-object-path stem flags :target-compile)))
+        (if (gethash object-path stems)
+            (error "duplicate stem ~S in *STEMS-AND-FLAGS*" stem)
+            (setf (gethash object-path stems) t)))
+    ;; Check for stupid typos in FLAGS list keywords.
     ;; FIXME: We should make sure that the :assem flag is only used
     ;; when paired with :not-host.
-    (let ((set-difference (set-difference flags *expected-stem-flags*)))
-      (when set-difference
-        (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
-               set-difference)))))
-  *stems-and-flags*)
+      (let ((set-difference (set-difference flags *expected-stem-flags*)))
+        (when set-difference
+          (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
+                 set-difference)))))
+  (cdr *stems-and-flags*))
 
 ;;;; tools to compile SBCL sources to create the cross-compiler
 
@@ -568,11 +633,17 @@
     (pathname obj)))
 (compile 'compile-stem)
 
+(defparameter *host-quirks*
+  (or #+cmu  '(:host-quirks-cmu :no-ansi-print-object)
+      #+ecl  '(:host-quirks-ecl)
+      #+sbcl '(:host-quirks-sbcl))) ; not so much a "quirk", but consistent anyway
+
 ;;; Execute function FN in an environment appropriate for compiling the
 ;;; cross-compiler's source code in the cross-compilation host.
 (defun in-host-compilation-mode (fn)
   (declare (type function fn))
-  (let ((*features* (cons :sb-xc-host *features*)))
+  (let ((sb-xc:*features* (append '(:sb-xc-host) *host-quirks* sb-xc:*features*))
+        (*readtable* *xc-readtable*))
     (funcall fn)))
 (compile 'in-host-compilation-mode)
 
@@ -629,3 +700,119 @@
              (funcall *target-compile-file* filename))))
 (compile 'target-compile-file)
 
+;;;; Floating-point number reader interceptor
+
+(defvar *choke-on-host-irrationals* t)
+(defun install-read-interceptor ()
+  ;; Intercept READ to catch inadvertent use of host floating-point literals.
+  ;; This prevents regressions in the portable float logic and allows passing
+  ;; characters to a floating-point library if we so choose.
+  ;; Only do this for new enough SBCL.
+  ;; DO-INSTANCE-TAGGED-SLOT was defined circa Nov 2014 and VERSION>= was defined
+  ;; ca. Nov 2013, but got moved from SB-IMPL or SB-C (inadvertently perhaps).
+  ;; It is not critical that this be enabled on all possible build hosts.
+  #+#.(cl:if (cl:and (cl:find-package "SB-C")
+                     (cl:find-symbol "SPLIT-VERSION-STRING" "SB-C")
+                     (cl:funcall (cl:find-symbol "VERSION>=" "SB-C")
+                                 (cl:funcall (cl:find-symbol "SPLIT-VERSION-STRING" "SB-C")
+                                             (cl:lisp-implementation-version))
+                                 '(1 4 6)))
+             '(and)
+             '(or))
+  (labels ((contains-irrational (x)
+             (typecase x
+               (cons
+                ;; Tail-recursion not guaranteed
+                (do ((cons x (cdr cons)))
+                    ((atom cons)
+                     (contains-irrational cons))
+                  (when (contains-irrational (car cons))
+                    (return t))))
+               (simple-vector (some #'contains-irrational x))
+               ;; We use package literals -- see e.g. SANE-PACKAGE - which
+               ;; must be treated as opaque, but COMMAs should not be opaque.
+               ;; There are also a few uses of "#.(find-layout)".
+               ;; However, the target-num objects should also be opaque
+               ;; and, testing for those types before the structure is defined
+               ;; is not fun. Other than moving the definitions into here
+               ;; from cross-early, there's no good way. But 'chill'
+               ;; should not define those structures.
+               ((and structure-object (not package))
+                (let ((type-name (string (type-of x))))
+                  ;; This "LAYOUT" refers to *our* object, not host-sb-kernel:layout.
+                  (unless (member type-name '("LAYOUT" "FLOAT" "COMPLEXNUM")
+                                  :test #'string=)
+                    ;(Format t "visit a ~/host-sb-ext:print-symbol-with-prefix/~%" (type-of x))
+                    ;; This generalizes over any structure. I need it because we
+                    ;; observe instances of SB-IMPL::COMMA and also HOST-SB-IMPL::COMMA.
+                    ;; (primordial-extensions get compiled before 'backq' is installed)
+                    (sb-kernel:do-instance-tagged-slot (i x)
+                      (when (contains-irrational (sb-kernel:%instance-ref x i))
+                        (return-from contains-irrational t))))))
+               ((or cl:complex cl:float)
+                x)))
+           (reader-intercept (f &optional stream (errp t) errval recursive)
+             (let* ((form (funcall f stream errp errval recursive))
+                    (bad-atom (and (not recursive) ; avoid checking inner forms
+                                   (not (eq form errval))
+                                   *choke-on-host-irrationals*
+                                   (contains-irrational form))))
+               (when bad-atom
+                 (setq *choke-on-host-irrationals* nil) ; one shot, otherwise tough to debug
+                 (error "Oops! didn't expect to read ~s containing ~s" form bad-atom))
+               form)))
+    (unless (sb-kernel:closurep (symbol-function 'read))
+      (sb-int:encapsulate 'read-preserving-whitespace 'protect #'reader-intercept)
+      (sb-int:encapsulate 'read 'protect #'reader-intercept)
+      (format t "~&; Installed READ interceptor~%"))))
+(compile 'install-read-interceptor)
+
+(defvar *math-ops-memoization* (make-hash-table :test 'equal))
+(defmacro with-math-journal (&body body)
+  `(let* ((table *math-ops-memoization*)
+          (memo (cons table (hash-table-count table))))
+     (assert (atom table)) ; prevent nested use of this macro
+     ;; Don't intercept READ until just-in-time, so that "chill" doesn't
+     ;; annoyingly get the interceptor installed.
+     (install-read-interceptor)
+     (let ((*math-ops-memoization* memo))
+       ,@body)
+     (when nil ; *compile-verbose*
+       (funcall (intern "SHOW-INTERNED-NUMBERS" "SB-IMPL") *standard-output*))
+     (when (> (hash-table-count table) (cdr memo))
+       (let ((filename "float-math.lisp-expr"))
+         (with-open-file (stream filename :direction :output
+                                          :if-exists :supersede)
+           (funcall (intern "DUMP-MATH-MEMOIZATION-TABLE" "SB-IMPL")
+                    table stream))
+         (format t "~&; wrote ~a - ~d entries"
+                 filename (hash-table-count table))))))
+
+;;;; Please avoid writing "consecutive" (un-nested) reader conditionals
+;;;; in this file, whether for the same or different feature test.
+;;;; The following example prints 3 different results in 3 different lisp
+;;;; implementations all of which have feature :linux (and not :nofeat).
+#|
+(let ((i 0))
+  (dolist (s '("#-nofeat #+linux a b c d e"
+               "#-nofeat #-linux a b c d e"
+               "#+nofeat #-linux a b c d e"
+               "#+nofeat #+linux a b c d e"
+               "#+linux #-nofeat a b c d e"
+               "#-linux #-nofeat a b c d e"
+               "#-linux #+nofeat a b c d e"
+               "#+linux #+nofeat a b c d e"
+               "#+linux #-linux a b c d e"
+               "#-linux #-linux a b c d e"
+               "#-linux #+linux a b c d e"
+               "#+linux #+linux a b c d e"
+               "#+nofeat #-nofeat a b c d e"
+               "#-nofeat #-nofeat a b c d e"
+               "#-nofeat #+nofeat a b c d e"
+               "#+nofeat #+nofeat a b c d e"))
+  (format t "test ~2d: ~a ~s~%"
+          (incf i)
+          (let ((stream (make-string-input-stream s)))
+            (list (read stream) (read stream) (read stream)))
+          s)))
+|#
