@@ -14,98 +14,130 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; Map of code-component -> list of PC offsets at which allocations occur.
 ;;; This table is needed in order to enable allocation profiling.
 (define-load-time-global *allocation-point-fixups*
   (make-hash-table :test 'eq :weakness :key :synchronized t))
 
-#!-x86-64
+#-x86-64
 (progn
 (defun convert-alloc-point-fixups (dummy1 dummy2)
   (declare (ignore dummy1 dummy2)))
-(defun sb!vm::statically-link-code-obj (code fixups)
+(defun sb-vm::statically-link-code-obj (code fixups)
   (declare (ignore code fixups))))
 
-(flet ((fixup (code-obj offset sym kind flavor layout-finder
-               preserved-lists statically-link-p)
+#+immobile-code
+(progn
+  ;; Use FDEFINITION because it strips encapsulations - whether that's
+  ;; the right behavior for it or not is a separate concern.
+  ;; If somebody tries (TRACE LENGTH) for example, it should not cause
+  ;; compilations to fail on account of LENGTH becoming a closure.
+  (defun sb-vm::function-raw-address (name &aux (fun (fdefinition name)))
+    (cond ((not (immobile-space-obj-p fun))
+           (error "Can't statically link to ~S: code is movable" name))
+          ((neq (fun-subtype fun) sb-vm:simple-fun-widetag)
+           (error "Can't statically link to ~S: non-simple function" name))
+          (t
+           (let ((addr (get-lisp-obj-address fun)))
+             (sap-ref-word (int-sap addr)
+                           (- (ash sb-vm:simple-fun-self-slot sb-vm:word-shift)
+                              sb-vm:fun-pointer-lowtag))))))
+
+  ;; Return the address to which to jump when calling FDEFN,
+  ;; which is either an fdefn or the name of an fdefn.
+  (defun sb-vm::fdefn-entry-address (fdefn)
+    (let ((fdefn (if (fdefn-p fdefn) fdefn (find-or-create-fdefn fdefn))))
+      (+ (get-lisp-obj-address fdefn)
+         (- 2 sb-vm:other-pointer-lowtag)))))
+
+;;; Point FUN's 'self' slot to FUN.
+;;; FUN must be pinned when calling this.
+(declaim (inline assign-simple-fun-self))
+(defun assign-simple-fun-self (fun)
+  (setf (%simple-fun-self fun)
+        ;; x86 backends store the address of the entrypoint in 'self'
+        #+(or x86 x86-64)
+        (%make-lisp-obj
+         (truly-the word (+ (get-lisp-obj-address fun)
+                            (ash sb-vm:simple-fun-insts-offset sb-vm:word-shift)
+                            (- sb-vm:fun-pointer-lowtag))))
+        ;; non-x86 backends store the function itself (what else?) in 'self'
+        #-(or x86 x86-64) fun))
+
+(flet ((fixup (code-obj offset sym kind flavor preserved-lists statically-link-p)
          (declare (ignorable statically-link-p))
          ;; PRESERVED-LISTS is a vector of lists of locations (by kind)
          ;; at which fixup must be re-applied after code movement.
          ;; CODE-OBJ must already be pinned in order to legally call this.
          ;; One call site that reaches here is below at MAKE-CORE-COMPONENT
          ;; and the other is LOAD-CODE, both of which pin the code.
-         (when (sb!vm:fixup-code-object
+         (when (sb-vm:fixup-code-object
                  code-obj offset
                  (ecase flavor
-                   ((:assembly-routine :assembly-routine*)
-                    (or (get-asm-routine sym (eq flavor :assembly-routine*))
-                        (error "undefined assembler routine: ~S" sym)))
+                   ((:assembly-routine :assembly-routine* :asm-routine-nil-offset)
+                    (- (or (get-asm-routine sym (eq flavor :assembly-routine*))
+                           (error "undefined assembler routine: ~S" sym))
+                       (if (eq flavor :asm-routine-nil-offset) sb-vm:nil-value 0)))
                    (:foreign (foreign-symbol-address sym))
                    (:foreign-dataref (foreign-symbol-address sym t))
                    (:code-object (get-lisp-obj-address code-obj))
-                   (:symbol-tls-index (ensure-symbol-tls-index sym))
-                   (:layout (get-lisp-obj-address (funcall layout-finder sym)))
-                   (:immobile-object (get-lisp-obj-address sym))
-                   #!+immobile-code
+                   #+sb-thread (:symbol-tls-index (ensure-symbol-tls-index sym))
+                   (:layout (get-lisp-obj-address
+                             (if (symbolp sym) (find-layout sym) sym)))
+                   (:immobile-symbol (get-lisp-obj-address sym))
+                   (:symbol-value (get-lisp-obj-address (symbol-global-value sym)))
+                   #+immobile-code
                    (:named-call
                     (when statically-link-p
                       (push (cons offset sym) (elt preserved-lists 3)))
-                    (sb!vm::fdefn-entry-address sym))
-                   #!+immobile-code (:static-call (sb!vm::function-raw-address sym)))
+                    (sb-vm::fdefn-entry-address sym))
+                   #+immobile-code (:static-call (sb-vm::function-raw-address sym)))
                  kind flavor)
            (ecase kind
              (:relative (push offset (elt preserved-lists 0)))
              (:absolute (push offset (elt preserved-lists 1)))))
          ;; These won't exist except for x86-64, but it doesn't matter.
-         (when (member sym '(sb!vm::enable-alloc-counter
-                             sb!vm::enable-sized-alloc-counter))
+         (when (member sym '(sb-vm::enable-alloc-counter
+                             sb-vm::enable-sized-alloc-counter))
            (push offset (elt preserved-lists 2))))
 
        (finish-fixups (code-obj preserved-lists)
          (declare (ignorable code-obj preserved-lists))
-         #!+(or immobile-space x86)
+         #+(or x86 x86-64)
          (let ((rel-fixups (elt preserved-lists 0))
                (abs-fixups (elt preserved-lists 1)))
            (when (or abs-fixups rel-fixups)
-             (setf (sb!vm::%code-fixups code-obj)
-                   (sb!c::pack-code-fixup-locs abs-fixups rel-fixups))))
+             (setf (sb-vm::%code-fixups code-obj)
+                   (sb-c::pack-code-fixup-locs abs-fixups rel-fixups))))
          (awhen (elt preserved-lists 2)
            (setf (gethash code-obj *allocation-point-fixups*)
                  (convert-alloc-point-fixups code-obj it)))
          (awhen (aref preserved-lists 3)
-           (sb!vm::statically-link-code-obj code-obj it))
+           (sb-vm::statically-link-code-obj code-obj it))
          ;; Assign all SIMPLE-FUN-SELF slots
          (dotimes (i (code-n-entries code-obj))
            (let ((fun (%code-entry-point code-obj i)))
-             (setf (%simple-fun-self fun)
-                   ;; x86 backends store the address of the entrypoint in 'self'
-                   #!+(or x86 x86-64)
-                   (%make-lisp-obj
-                    (truly-the word (+ (get-lisp-obj-address fun)
-                                       (ash sb!vm:simple-fun-code-offset sb!vm:word-shift)
-                                       (- sb!vm:fun-pointer-lowtag))))
-                   ;; non-x86 backends store the function itself (what else?) in 'self'
-                   #!-(or x86 x86-64) fun)
+             (assign-simple-fun-self fun)
              ;; And maybe store the layout in the high half of the header
-             #!+(and compact-instance-header x86-64)
+             #+(and compact-instance-header x86-64)
              (setf (sap-ref-32 (int-sap (get-lisp-obj-address fun))
-                               (- 4 sb!vm:fun-pointer-lowtag))
+                               (- 4 sb-vm:fun-pointer-lowtag))
                    (truly-the (unsigned-byte 32)
                      (get-lisp-obj-address #.(find-layout 'function))))))
          ;; And finally, make the memory range executable
-         #!-(or x86 x86-64)
-         (sb!vm:sanctify-for-execution code-obj)))
+         #-(or x86 x86-64)
+         (sb-vm:sanctify-for-execution code-obj)))
 
-  (defun apply-fasl-fixups (fop-stack code-obj &aux (top (svref fop-stack 0)))
+  (defun apply-fasl-fixups (fop-stack code-obj n-fixups &aux (top (svref fop-stack 0)))
     (dx-let ((preserved (make-array 4 :initial-element nil)))
       (macrolet ((pop-fop-stack () `(prog1 (svref fop-stack top) (decf top))))
-        (dotimes (i (pop-fop-stack) (setf (svref fop-stack 0) top))
+        (dotimes (i n-fixups (setf (svref fop-stack 0) top))
           (multiple-value-bind (offset kind flavor)
-              (sb!fasl::!unpack-fixup-info (pop-fop-stack))
+              (sb-fasl::!unpack-fixup-info (pop-fop-stack))
             (fixup code-obj offset (pop-fop-stack) kind flavor
-                   #'find-layout preserved nil))))
+                   preserved nil))))
       (finish-fixups code-obj preserved)))
 
   (defun apply-core-fixups (fixup-notes code-obj)
@@ -118,14 +150,37 @@
                  (fixup-name fixup)
                  (fixup-note-kind note)
                  (fixup-flavor fixup)
-               ;; Compiling to memory creates layout fixups with the name being
-               ;; an instance of LAYOUT, not a symbol. Those probably should be
-               ;; :IMMOBILE-OBJECT fixups. But since they're not, inform the
-               ;; fixupper not to call find-layout on them.
-                 #'identity
                  preserved t)))
       (finish-fixups code-obj preserved))))
 
+;;; Return a behaviorally identical copy of CODE.
+(defun copy-code-object (code)
+  ;; Must have one simple-fun
+  (aver (= (code-n-entries code) 1))
+  ;; Disallow relative instruction operands.
+  ;; (This restriction could be removed by actually performing fixups)
+  ;; x86-64 absolute fixups are OK since they will only point to static objects.
+  #+x86-64
+  (aver (not (nth-value
+              1 (sb-c:unpack-code-fixup-locs (sb-vm::%code-fixups code)))))
+  (let* ((nbytes (code-object-size code))
+         (boxed (code-header-words code)) ; word count
+         (unboxed (- nbytes (ash boxed sb-vm:word-shift))) ; byte count
+         (copy (allocate-code-object :dynamic boxed unboxed)))
+    (with-pinned-objects (code copy)
+      (%byte-blt (code-instructions code) 0 (code-instructions copy) 0 unboxed)
+      ;; copy boxed constants so that the fixup step (if needed) sees the 'fixups'
+      ;; slot from the new object.
+      (loop for i from 2 below boxed
+            do (setf (code-header-ref copy i) (code-header-ref code i)))
+      ;; x86 needs to fixup instructions that reference code constants,
+      ;; and the jmp to TAIL-CALL-VARIABLE
+      #+x86 (alien-funcall (extern-alien "gencgc_apply_code_fixups" (function void unsigned unsigned))
+                           (- (get-lisp-obj-address code) sb-vm:other-pointer-lowtag)
+                           (- (get-lisp-obj-address copy) sb-vm:other-pointer-lowtag))
+      (assign-simple-fun-self (%code-entry-point copy 0)))
+    copy))
+
 ;;; Note the existence of FUNCTION.
 (defun note-fun (info function object)
   (declare (type function function)
@@ -161,7 +216,7 @@
   (let ((debug-info (debug-info-for-component component)))
     (let* ((2comp (component-info component))
            (constants (ir2-component-constants 2comp))
-           (nboxed (align-up (length constants) sb!c::code-boxed-words-align))
+           (nboxed (align-up (length constants) sb-c::code-boxed-words-align))
            (code-obj (allocate-code-object
                       (component-mem-space component) nboxed length)))
 
@@ -180,18 +235,23 @@
       (let* ((entries (ir2-component-entries 2comp))
              (fun-index (length entries)))
         (dolist (entry-info entries)
-          (let ((fun (%code-entry-point code-obj (decf fun-index))))
-            (setf (%simple-fun-name fun) (entry-info-name entry-info))
-            (setf (%simple-fun-arglist fun) (entry-info-arguments entry-info))
-            (setf (%simple-fun-type fun) (entry-info-type entry-info))
-            (apply #'set-simple-fun-info fun
-                   (entry-info-form/doc/xrefs entry-info))
+          (let ((fun (%code-entry-point code-obj (decf fun-index)))
+                (w (+ sb-vm:code-constants-offset
+                      (* sb-vm:code-slots-per-simple-fun fun-index))))
+            (setf (code-header-ref code-obj (+ w sb-vm:simple-fun-name-slot))
+                  (entry-info-name entry-info)
+                  (code-header-ref code-obj (+ w sb-vm:simple-fun-arglist-slot))
+                  (entry-info-arguments entry-info)
+                  (code-header-ref code-obj (+ w sb-vm:simple-fun-source-slot))
+                  (entry-info-form/doc entry-info)
+                  (code-header-ref code-obj (+ w sb-vm:simple-fun-info-slot))
+                  (entry-info-type/xref entry-info))
             (note-fun entry-info fun object))))
 
       (push debug-info (core-object-debug-info object))
       (setf (%code-debug-info code-obj) debug-info)
 
-      (do ((index sb!vm:code-constants-offset (1+ index)))
+      (do ((index sb-vm:code-constants-offset (1+ index)))
           ((>= index (length constants)))
         (let ((const (aref constants index)))
             (etypecase const
