@@ -68,7 +68,7 @@
         (pushnew 2block (car (gethash new *2block-info*)))))))
 
 ;;;; Conditional move insertion support code
-#!-sb-fluid (declaim (inline vop-name))
+#-sb-fluid (declaim (inline vop-name))
 (defun vop-name (vop &optional default)
   (declare (type vop vop))
   (let ((vop-info (vop-info vop)))
@@ -272,8 +272,13 @@
   (flet ((start-vop (block)
            (do ((block block (ir2-block-next block)))
                ((null block) nil)
-             (when (ir2-block-start-vop block)
-               (return (ir2-block-start-vop block)))))
+             (let ((vop (ir2-block-start-vop block)))
+               (when vop
+                 (if (eq (vop-name vop) 'sb-c:note-environment-start)
+                     (let ((next (vop-next vop)))
+                       (when next
+                         (return next)))
+                     (return vop))))))
          (next-label (block)
            (do ((block (ir2-block-next block)
                   (ir2-block-next block)))
@@ -360,11 +365,107 @@
                                     (start-vop (ir2-block-next block))))))))
         (delete-fall-through-jumps component)))))
 
+(defmacro do-vops (vop ir2-block &body body)
+  `(do ((,vop (ir2-block-start-vop ,ir2-block) (vop-next ,vop)))
+       ((null ,vop))
+     ,@body))
+
+(defun next-vop (vop)
+  (or (vop-next vop)
+      (let ((next-block (ir2-block-next (vop-block vop))))
+        (and (not (or (ir2-block-%trampoline-label next-block)
+                      (ir2-block-%label next-block)))
+             (ir2-block-start-vop next-block)))))
+
+(defun immediate-templates (fun &optional (constants t))
+  (let ((primitive-types (list (primitive-type-or-lose 'character)
+                               (primitive-type-or-lose 'fixnum)
+                               (primitive-type-or-lose 'sb-vm::positive-fixnum)
+                               (primitive-type-or-lose 'double-float)
+                               (primitive-type-or-lose 'single-float)
+                               .
+                               #+(or 64-bit 64-bit-registers)
+                               ((primitive-type-or-lose 'sb-vm::unsigned-byte-63)
+                                (primitive-type-or-lose 'sb-vm::unsigned-byte-64)
+                                (primitive-type-or-lose 'sb-vm::signed-byte-64))
+                               #-(or 64-bit 64-bit-registers)
+                               ((primitive-type-or-lose 'sb-vm::unsigned-byte-31)
+                                (primitive-type-or-lose 'sb-vm::unsigned-byte-32)
+                                (primitive-type-or-lose 'sb-vm::signed-byte-32)))))
+    (loop for template in (fun-info-templates (fun-info-or-lose fun))
+          when (and (typep (template-result-types template) '(cons (eql :conditional)))
+                    (loop for type in (template-arg-types template)
+                          always (and (consp type)
+                                      (case (car type)
+                                        (:or
+                                         (loop for type in (cdr type)
+                                               always (memq type primitive-types)))
+                                        (:constant constants)))))
+          collect (template-name template))))
+
+(define-load-time-global *comparison-vops*
+    (append (immediate-templates 'eq)
+            (immediate-templates '=)
+            (immediate-templates '>)
+            (immediate-templates '<)
+            (immediate-templates 'char<)
+            (immediate-templates 'char>)
+            (immediate-templates 'char=)))
+
+(define-load-time-global *commutative-comparison-vops*
+    (append (immediate-templates 'eq nil)
+            (immediate-templates 'char= nil)
+            (immediate-templates '= nil)))
+
+(defun vop-arg-list (vop)
+  (let ((args (loop for arg = (vop-args vop) then (tn-ref-across arg)
+                    while arg
+                    collect (tn-ref-tn arg))))
+    (if (vop-codegen-info vop)
+        (nconc args (vop-codegen-info vop))
+        args)))
+
+(defun vop-args-equal (vop1 vop2 &optional reverse)
+  (let* ((args1 (vop-arg-list vop1))
+         (args2 (vop-arg-list vop2)))
+    (equal (if reverse
+               (reverse args1)
+               args1)
+           args2)))
+
+;;; Turn CMP X,Y BRANCH-IF M CMP X,Y BRANCH-IF N
+;;; into CMP X,Y BRANCH-IF M BRANCH-IF N
+(defun ir2-optimize-comparisons (component)
+  (do-ir2-blocks (block component)
+    (do-vops branch-if block
+      (when (eq (vop-name branch-if) 'branch-if)
+        (let ((prev (vop-prev branch-if)))
+          (when (and prev
+                     (memq (vop-name prev) *comparison-vops*))
+            (let ((next (next-vop branch-if))
+                  transpose)
+              (when (and next
+                         (memq (vop-name next) *comparison-vops*)
+                         (or (vop-args-equal prev next)
+                             (and (or (setf transpose
+                                            (memq (vop-name prev) *commutative-comparison-vops*))
+                                      (memq (vop-name next) *commutative-comparison-vops*))
+                                  (vop-args-equal prev next t))))
+                (when transpose
+                  ;; Could flip the flags for non-commutative operations
+                  (loop for tn-ref = (vop-args prev) then (tn-ref-across tn-ref)
+                        for arg in (nreverse (vop-arg-list prev))
+                        do (change-tn-ref-tn tn-ref arg)))
+                (delete-vop next)))))))))
+
 (defun ir2-optimize (component)
   (let ((*2block-info*  (make-hash-table :test #'eq)))
     (initialize-ir2-blocks-flow-info component)
 
     (convert-cmovs component)
+    #+x86-64 ;; while it's portable the VOPs are not validated for
+             ;; compatibility on other backends yet.
+    (ir2-optimize-comparisons component)
     (delete-unused-ir2-blocks component))
 
   (values))
