@@ -66,7 +66,7 @@ static struct thread *postmortem_thread;
 
 #endif
 
-int dynamic_values_bytes=TLS_SIZE*sizeof(lispobj);  /* same for all threads */
+int dynamic_values_bytes = 4096 * sizeof(lispobj);  // same for all threads
 struct thread *all_threads;
 
 #ifdef LISP_FEATURE_SB_THREAD
@@ -76,9 +76,11 @@ static pthread_mutex_t create_thread_lock = PTHREAD_MUTEX_INITIALIZER;
 
 #ifdef LISP_FEATURE_GCC_TLS
 __thread struct thread *current_thread;
-#endif
+__thread int is_lisp_thread;
+#else
 pthread_key_t lisp_thread = 0;
 #endif
+#endif
 
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
 extern lispobj call_into_lisp_first_time(lispobj fun, lispobj *args, int nargs)
@@ -194,9 +196,14 @@ initial_thread_trampoline(struct thread *th)
     lispobj *args = NULL;
 #endif
 #ifdef LISP_FEATURE_SB_THREAD
+# ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 1;
+# else
     pthread_setspecific(lisp_thread, (void *)1);
+# endif
 #endif
-#if defined(THREADS_USING_GCSIGNAL) && (defined(LISP_FEATURE_PPC) || defined(LISP_FEATURE_ARM64))
+#if defined THREADS_USING_GCSIGNAL && \
+    (defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || defined LISP_FEATURE_ARM64)
     /* SIG_STOP_FOR_GC defaults to blocked on PPC? */
     unblock_gc_signals(0,0);
 #endif
@@ -343,10 +350,14 @@ init_new_thread(struct thread *th,
 {
     int lock_ret;
 
+#ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 1;
+#else
     pthread_setspecific(lisp_thread, (void *)1);
+#endif
     if(arch_os_thread_init(th)==0) {
         /* FIXME: handle error */
-        lose("arch_os_thread_init failed\n");
+        lose("arch_os_thread_init failed");
     }
 
     th->os_thread=thread_self();
@@ -511,7 +522,7 @@ attach_os_thread(init_thread_data *scribble)
     block_deferrable_signals(&scribble->oldset);
 
 #ifndef LISP_FEATURE_SB_SAFEPOINT
-    /* initial-thread-function-trampoline doesn't like when the GC signal is blocked */
+    /* new-lisp-thread-trampoline doesn't like when the GC signal is blocked */
     /* FIXME: could be done using a single call to pthread_sigmask
        together with locking the deferrable signals above. */
     unblock_gc_signals(0, 0);
@@ -534,7 +545,7 @@ attach_os_thread(init_thread_data *scribble)
   stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
 #elif defined(LISP_FEATURE_DARWIN)
     stack_size = pthread_get_stacksize_np(os);
-    stack_addr = pthread_get_stackaddr_np(os) - stack_size;
+    stack_addr = (char*)pthread_get_stackaddr_np(os) - stack_size;
 #else
     pthread_attr_t attr;
 #ifdef LISP_FEATURE_FREEBSD
@@ -568,7 +579,11 @@ detach_os_thread(init_thread_data *scribble)
     undo_init_new_thread(th, scribble);
 
     odxprint(misc, "deattach_os_thread: detached");
+#ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 0;
+#else
     pthread_setspecific(lisp_thread, (void *)0);
+#endif
     thread_sigmask(SIG_SETMASK, &scribble->oldset, 0);
     free_thread_struct(th);
 }
@@ -671,7 +686,7 @@ callback_wrapper_trampoline(
  */
 
 static struct thread *
-create_thread_struct(lispobj initial_function) {
+create_thread_struct(lispobj start_routine) {
 #if defined(LISP_FEATURE_SB_THREAD) || defined(LISP_FEATURE_WIN32)
     unsigned int i;
 #endif
@@ -684,7 +699,7 @@ create_thread_struct(lispobj initial_function) {
      * on the alignment passed from os_validate, since that might
      * assume the current (e.g. 4k) pagesize, while we calculate with
      * the biggest (e.g. 64k) pagesize allowed by the ABI. */
-    void *spaces = os_validate(IS_THREAD_STRUCT, NULL, THREAD_STRUCT_SIZE);
+    void *spaces = os_validate(MOVABLE|IS_THREAD_STRUCT, NULL, THREAD_STRUCT_SIZE);
     if(!spaces)
         return NULL;
     /* Aligning up is safe as THREAD_STRUCT_SIZE has
@@ -702,19 +717,20 @@ create_thread_struct(lispobj initial_function) {
 
 #ifdef LISP_FEATURE_SB_THREAD
     lispobj* tls = (lispobj*)th;
-    for(i = 0; i < TLS_SIZE; i++)
+    for(i = 0; i < (unsigned int)(dynamic_values_bytes/N_WORD_BYTES); i++)
         tls[i] = NO_TLS_VALUE_MARKER_WIDETAG;
+    th->tls_size = dynamic_values_bytes;
+#endif
+    uword_t* __attribute__((__unused__)) constants = (uword_t*)th;
+#if defined LISP_FEATURE_X86_64 && defined LISP_FEATURE_LINUX
+    constants[THREAD_MSAN_XOR_CONSTANT_SLOT] = 0x500000000000;
 #endif
 #ifdef LISP_FEATURE_GENCGC
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#ifdef THREAD_VARYOBJ_CARD_MARKS_SLOT
     extern unsigned int* varyobj_page_touched_bits;
-    th->varyobj_space_addr  = VARYOBJ_SPACE_START;
-    th->varyobj_card_count  = varyobj_space_size / IMMOBILE_CARD_BYTES;
-    th->varyobj_card_marks  = (lispobj)varyobj_page_touched_bits;
-#else
-    th->varyobj_space_addr  = 0;
-    th->varyobj_card_count  = 0;
-    th->varyobj_card_marks  = 0;
+    constants[THREAD_VARYOBJ_SPACE_ADDR_SLOT] = VARYOBJ_SPACE_START;
+    constants[THREAD_VARYOBJ_CARD_COUNT_SLOT] = varyobj_space_size / IMMOBILE_CARD_BYTES;
+    constants[THREAD_VARYOBJ_CARD_MARKS_SLOT] = (lispobj)varyobj_page_touched_bits;
 #endif
     th->dynspace_addr       = DYNAMIC_SPACE_START;
     th->dynspace_card_count = page_table_pages;
@@ -733,7 +749,7 @@ create_thread_struct(lispobj initial_function) {
     th->this=th;
     th->os_thread=0;
 #if defined(LAYOUT_OF_FUNCTION) && defined(LISP_FEATURE_SB_THREAD)
-    th->function_layout = LAYOUT_OF_FUNCTION << 32;
+    constants[THREAD_FUNCTION_LAYOUT_SLOT] = LAYOUT_OF_FUNCTION << 32;
 #endif
     // Once allocated, the allocation profiling buffer sticks around.
     // If present and enabled, assign into the new thread.
@@ -851,7 +867,7 @@ create_thread_struct(lispobj initial_function) {
      * mess up th->no_tls_value_marker. Fail now if that happened. */
     gc_assert(th->no_tls_value_marker == NO_TLS_VALUE_MARKER_WIDETAG);
 #endif
-    th->no_tls_value_marker=initial_function;
+    th->no_tls_value_marker = start_routine;
 
 #if defined(LISP_FEATURE_WIN32)
     for (i = 0; i<sizeof(th->private_events.events)/
@@ -866,12 +882,12 @@ create_thread_struct(lispobj initial_function) {
 
 void create_initial_thread(lispobj initial_function) {
     struct thread *th = create_thread_struct(initial_function);
-#ifdef LISP_FEATURE_SB_THREAD
+#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_GCC_TLS)
     pthread_key_create(&lisp_thread, 0);
 #endif
     if(th) {
         initial_thread_trampoline(th); /* no return */
-    } else lose("can't create initial thread\n");
+    } else lose("can't create initial thread");
 }
 
 #ifdef LISP_FEATURE_SB_THREAD
@@ -898,7 +914,7 @@ boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
     pthread_attr_t attr;
     if (pthread_attr_init(&attr) == 0) {
 
-    /* See perform_thread_post_mortem for at least one reason why this lock is neccessary */
+    /* See perform_thread_post_mortem for at least one reason why this lock is necessary */
         retcode = pthread_mutex_lock(&create_thread_lock);
         gc_assert(retcode == 0);
 
@@ -910,13 +926,19 @@ boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
             (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline_switch_stack, th))
 #else
 
-# if defined(LISP_FEATURE_WIN32)
+# ifdef LISP_FEATURE_WIN32
             pthread_attr_setstacksize(&attr, thread_control_stack_size) ||
 # elif defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
             pthread_attr_setstack(&attr, th->control_stack_start, thread_control_stack_size) ||
 # else
             pthread_attr_setstack(&attr, th->alien_stack_start, ALIEN_STACK_SIZE) ||
 # endif
+# ifdef LISP_FEATURE_NETBSD
+            /* Even though the manpage says pthread_attr_setstack
+               would override the guard page, it's no longer true. */
+            pthread_attr_setguardsize(&attr, 0) ||
+# endif
+
             (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline, th))
 #endif
             ) {
@@ -933,20 +955,20 @@ boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
     return success;
 }
 
-os_thread_t create_thread(lispobj initial_function) {
+os_thread_t create_thread(lispobj start_routine) {
     struct thread *th, *thread = arch_os_get_current_thread();
     os_thread_t kid_tid = 0;
 
     /* Must defend against async unwinds. */
     if (read_TLS(INTERRUPTS_ENABLED, thread) != NIL)
-        lose("create_thread is not safe when interrupts are enabled.\n");
+        lose("create_thread is not safe when interrupts are enabled.");
 
     /* Assuming that a fresh thread struct has no lisp objects in it,
      * linking it to all_threads can be left to the thread itself
-     * without fear of gc lossage. initial_function violates this
+     * without fear of gc lossage. 'start_routine' violates this
      * assumption and must stay pinned until the child starts up. */
-    th = create_thread_struct(initial_function);
-    if (th && !create_os_thread(th,&kid_tid)) {
+    th = create_thread_struct(start_routine);
+    if (th && !create_os_thread(th, &kid_tid)) {
         free_thread_struct(th);
         kid_tid = 0;
     }
@@ -1114,7 +1136,10 @@ kill_safely(os_thread_t os_thread, int signal)
          * would go wrong.  Why are we running interruptions while
          * stopping the world though?  Test case is (:ASYNC-UNWIND
          * :SPECIALS), especially with s/10/100/ in both loops. */
-        if (os_thread == pthread_self()) {
+        /* From the linux man page on pthread_self() -
+         * "variables  of  type  pthread_t  can't  portably be compared using
+         *  the C equality operator (==); use pthread_equal(3) instead." */
+        if (thread_equal(os_thread, pthread_self())) {
             pthread_kill(os_thread, signal);
 #ifdef LISP_FEATURE_WIN32
             check_pending_thruptions(NULL);
@@ -1130,7 +1155,7 @@ kill_safely(os_thread_t os_thread, int signal)
             if (thread->os_thread == os_thread) {
                 int status = pthread_kill(os_thread, signal);
                 if (status)
-                    lose("kill_safely: pthread_kill failed with %d\n", status);
+                    lose("kill_safely: pthread_kill failed with %d", status);
 #if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THRUPTION)
                 wake_thread_win32(thread);
 #endif
@@ -1148,7 +1173,7 @@ kill_safely(os_thread_t os_thread, int signal)
 #else
         int status;
         if (os_thread != 0)
-            lose("kill_safely: who do you want to kill? %d?\n", os_thread);
+            lose("kill_safely: who do you want to kill? %d?", os_thread);
         /* Dubious (as in don't know why it works) workaround for the
          * signal sometimes not being generated on darwin. */
 #ifdef LISP_FEATURE_DARWIN
@@ -1164,7 +1189,7 @@ kill_safely(os_thread_t os_thread, int signal)
         if (status == 0) {
             return 0;
         } else {
-            lose("cannot raise signal %d, %d %s\n",
+            lose("cannot raise signal %d, %d %s",
                  signal, status, strerror(errno));
         }
 #endif
